@@ -104,10 +104,35 @@ def upload_coredump(ctx: MemfaultCliClickContext, path: str):
     required=True,
     help="The unique identifier of a device",
 )
+@click.option(
+    "--attribute",
+    type=click.Tuple([str, str]),
+    multiple=True,
+    help="""
+    Attribute associated with the uploaded file. Multiple attributes can be passed.
+    The value argument is attempted to be interpreted as a JSON string first, if that
+    fails, the value is interpreted as a string value as-is. Note however that only
+    number, boolean, string or null values are allowed (objects and arrays are not allowed).
+
+    Examples:
+
+    Key "my_int" with numeric value 1234:
+
+    --attribute my_int 1234
+
+    Key "my_string" with string value 'hello':
+
+    --attribute my_string hello
+
+    Key "my_bool" with boolean true value:
+
+    --attribute my_bool true
+    """,
+)
 @click_argument_path
 @pass_memfault_cli_context
 def upload_xed(ctx: MemfaultCliClickContext, path: str, **kwargs):
-    """Upload an .xed file for analysis."""
+    """Upload an .xed or .xcd file for analysis."""
     ctx.obj.update(**kwargs)
 
     _do_upload_or_raise(ctx, path, XedUploader)
@@ -283,13 +308,27 @@ def upload_android_app_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs
 @cli.command(name="upload-ota-payload")
 @click.option("--hardware-version", required=True)
 @click.option("--software-type", required=True)
-@click.option("--software-version", required=True)
+@click.option(
+    "--software-version",
+    required=False,
+    help="Make the OTA Payload a Full Release payload for this version.",
+)
+@click.option(
+    "--delta-from",
+    required=False,
+    help="Pass --delta-from FROM_VERSION make the OTA Payload a Delta Release payload from the version FROM_VERSION. Use alongside --delta-target TARGET_VERSION",
+)
+@click.option(
+    "--delta-target",
+    required=False,
+    help="Pass --delta-target TARGET_VERSION to make the OTA Payload a Delta Release payload target version TARGET_VERSION. Use alongside --delta-from FROM_VERSION.",
+)
 @click.option("--notes", default="", help="Optional release notes.")
 @click.option(
     "--must-pass-through",
     required=False,
     is_flag=True,
-    help="When the release is deployed to a Cohort, forces a device to update through this version even if a newer version has also been deployed to the Cohort.",
+    help="When the Release is deployed to a Cohort, forces a device to update through this version even if a newer version has also been deployed to the Cohort.",
 )
 @click_option_revision
 @click_argument_path
@@ -297,8 +336,8 @@ def upload_android_app_symbols(ctx: MemfaultCliClickContext, path: str, **kwargs
 def upload_ota_payload(ctx: MemfaultCliClickContext, path: str, **kwargs):
     """Upload a binary to be used for an OTA update.
 
-    See https://mflt.io/34PyNGQ for details about 'hardware-version', 'software-type' and
-    'software-version' nomenclature.
+    See https://mflt.io/34PyNGQ for details about 'hardware-version',
+    'software-type' and 'software-version' nomenclature.
 
     When deployed, this is the binary that will be returned from the Memfault /latest endpoint
     which can be used for an Over The Air (OTA) update.
@@ -315,14 +354,32 @@ def upload_ota_payload(ctx: MemfaultCliClickContext, path: str, **kwargs):
             --revision 89335ffade90ff7697e2ce5238bd4c68978b6d6e \\
             build/stm32-fw.bin
 
-        Reference: https://mflt.io/create-release
+    \b
+    Example Delta OTA Upload:
+    \b
+        $ memfault --org-token $ORG_TOKEN \\
+                   --org acme-inc --project smart-sink \\
+            upload-ota-payload \\
+            --hardware-version mp \\
+            --software-type stm32-fw \\
+            --delta-from 0.9.0 \\
+            --delta-target 1.0.0-alpha \\
+            --revision 89335ffade90ff7697e2ce5238bd4c68978b6d6e \\
+            build/stm32-fw-delta.bin
+
+    Reference: https://mflt.io/create-release
     """
     ctx.obj.update(**kwargs)
+    ctx.check_required_either(
+        {"software_version"}, {"delta_from", "delta_target"}, mutually_exclusive=True
+    )
     _do_upload_or_raise(ctx, path, ReleaseArtifactUploader)
 
 
 @cli.command(name="deploy-release")
-@click.option("--release-version", type=str, required=True)
+@click.option("--release-version", type=str, required=False)
+@click.option("--delta-from", required=False)
+@click.option("--delta-target", required=False)
 @click.option("--cohort", type=str, required=True)
 @click.option(
     "--rollout-percent",
@@ -332,9 +389,7 @@ def upload_ota_payload(ctx: MemfaultCliClickContext, path: str, **kwargs):
     help="The (randomly sampled) percentage of devices in the Cohort to rollout the release to.",
 )
 @pass_memfault_cli_context
-def deploy_release(
-    ctx: MemfaultCliClickContext, release_version: str, cohort: str, rollout_percent: int, **kwargs
-):
+def deploy_release(ctx: MemfaultCliClickContext, cohort: str, rollout_percent: int, **kwargs):
     """Publish a Release to a Cohort.
 
     \b
@@ -349,9 +404,15 @@ def deploy_release(
     ctx.obj.update(**kwargs)
 
     ctx.check_required_cli_args(authenticator=ctx.authenticator)
-
+    ctx.check_required_either(
+        {"release_version"}, {"delta_from", "delta_target"}, mutually_exclusive=True
+    )
+    release_version = ctx.obj.get("release_version") or (
+        ctx.obj.get("delta_from"),
+        ctx.obj.get("delta_target"),
+    )
     deployer = Deployer(ctx=ctx, authenticator=ctx.authenticator)
-    deployer.deploy(release_version=release_version, cohort=cohort, rollout_percent=rollout_percent)
+    deployer.deploy(cohort=cohort, release_version=release_version, rollout_percent=rollout_percent)
 
 
 def _do_post_chunks_or_raise(ctx: MemfaultCliClickContext, chunks: List[bytes]):
