@@ -1,10 +1,14 @@
-from typing import Optional
+import json
+import logging
+from typing import Iterator, List, Optional, Set, Union
 
 import click
 
 from memfault_cli.authenticator import Authenticator
 from memfault_cli.functools_ext import cached_property
 
+LOG = logging.getLogger(__name__)
+
 
 class PackageInfo:  # noqa: B903 - no data classes in Python 3.6
     def __init__(self, name: str, version_name: str, version_code: str):
@@ -13,13 +17,63 @@ class PackageInfo:  # noqa: B903 - no data classes in Python 3.6
         self.version_code = version_code
 
 
-class SoftwareInfo:  # noqa: B903 - no data classes in Python 3.6
-    def __init__(self, software_type: str, software_version: str, revision: Optional[str] = None):
-        self.software_type = software_type
+class FullReleaseVersion:  # noqa: B903 - no data classes in Python 3.6
+    def __init__(self, software_version: str):
         self.software_version = software_version
+
+    def to_json_representable(self):
+        return self.software_version
+
+
+class DeltaReleaseVersion:  # noqa: B903 - no data classes in Python 3.6
+    def __init__(self, delta_from: str, delta_target: str):
+        self.delta_from = delta_from
+        self.delta_target = delta_target
+
+    def to_json_representable(self):
+        return [self.delta_from, self.delta_target]
+
+
+class SoftwareInfo:  # noqa: B903 - no data classes in Python 3.6
+    def __init__(
+        self,
+        software_type: str,
+        version: Union[FullReleaseVersion, DeltaReleaseVersion],
+        revision: Optional[str] = None,
+    ):
+        self.software_type = software_type
+        self.version = version
         self.revision = revision
 
 
+CustomMetricReadingValueType = Union[int, float, str, bool]
+
+
+class Attribute:  # noqa: B903 - no data classes in Python 3.6
+    def __init__(self, string_key: str, value: CustomMetricReadingValueType) -> None:
+        self.string_key = string_key
+        self.value = value
+
+    @staticmethod
+    def from_cli_args(key: str, cli_value: str):
+        return Attribute(string_key=key, value=Attribute._convert_value(cli_value))
+
+    @staticmethod
+    def _convert_value(cli_value: str) -> CustomMetricReadingValueType:
+        if cli_value == "":
+            return None
+        try:
+            value = json.loads(cli_value)
+        except json.decoder.JSONDecodeError:
+            value = cli_value
+        if value is not None and not isinstance(value, (int, float, str, bool)):
+            LOG.warning(
+                "Only boolean, number, string or null types are allowed! Treating passed value as a string."
+            )
+            return cli_value
+        return value
+
+
 class XedInfo:  # noqa: B903 - no data classes in Python 3.6
     def __init__(self, device_serial: str, hardware_version: str):
         self.device_serial = device_serial
@@ -40,11 +94,13 @@ class MemfaultCliClickContext(object):
     def __init__(self) -> None:
         self.obj = {}
 
+    def _format_as_option(self, property_name: str) -> str:
+        return f"--{property_name.replace('_', '-')}"
+
     def _find_obj_or_raise(self, name):
         value = self.obj.get(name)
         if value is None:
-            option = name.replace("_", "-")
-            raise click.exceptions.UsageError(f'Missing option "--{option}".')
+            raise click.exceptions.UsageError(f"Missing option {self._format_as_option(name)!r}.")
         return value
 
     def check_required_cli_args(self, *, authenticator: Authenticator):
@@ -53,6 +109,33 @@ class MemfaultCliClickContext(object):
         for arg in required_args:
             self._find_obj_or_raise(arg)
 
+    def _format_arg_set(self, arg_set: Iterator[Set[str]]) -> str:
+        return ", ".join([f"{self._format_as_option(arg)!r}" for arg in sorted(arg_set)])
+
+    def check_required_either(self, *args: Set[str], mutually_exclusive=False):
+        """ Check that any full set of args is present """
+        valid_arg_sets = []
+        passed_arg_sets = []
+
+        for arg_set in args:
+            args_present = [self.obj.get(arg, None) is not None for arg in arg_set]
+            if any(args_present):
+                passed_arg_sets.append(arg_set)
+            if all(args_present):
+                valid_arg_sets.append(arg_set)
+
+        if mutually_exclusive and len(passed_arg_sets) > 1:
+            formatted_passed_args = [self._format_arg_set(arg_set) for arg_set in passed_arg_sets]
+            raise click.exceptions.UsageError(
+                f"Parameters {' and '.join(sorted(formatted_passed_args))} may not be used together."
+            )
+
+        if len(valid_arg_sets) == 0:
+            formatted_required_args = [self._format_arg_set(arg_set) for arg_set in args]
+            raise click.exceptions.UsageError(
+                f"Please pass either {' or '.join(sorted(formatted_required_args))}."
+            )
+
     @cached_property
     def authenticator(self) -> Authenticator:
         return Authenticator.create_authenticator_given_context_or_raise(self)
@@ -90,21 +173,37 @@ class MemfaultCliClickContext(object):
         sw_type = self.obj.get("software_type")
         sw_ver = self.obj.get("software_version")
         revision = self.obj.get("revision")
+        delta_from = self.obj.get("delta_from")
+        delta_target = self.obj.get("delta_target")
 
-        if revision and (sw_ver is None or sw_type is None):
+        version = None
+        if delta_from and delta_target:
+            version = DeltaReleaseVersion(delta_from, delta_target)
+        elif sw_ver:
+            version = FullReleaseVersion(sw_ver)
+
+        if revision and (version is None or sw_type is None):
             raise click.exceptions.UsageError(
-                '"--software-version" and "--software-type" must be specified when using --revision'
+                "A version ('--software-version' or '--delta-{{from,target}}') and '--software-type' must be specified when using '--revision'"
             )
 
-        if sw_type is None and sw_ver is None:
+        if sw_type is None and version is None:
             return None
 
-        if sw_type is None or sw_ver is None:
+        if sw_type is None or version is None:
             raise click.exceptions.UsageError(
-                '"--software-version" and "--software-type" must be specified together'
+                "Version ('--software-version' or '--delta-{{from,target}}') and '--software-type' must be specified together"
             )
 
-        return SoftwareInfo(software_type=sw_type, software_version=sw_ver, revision=revision)
+        return SoftwareInfo(
+            software_type=sw_type,
+            version=version,
+            revision=revision,
+        )
+
+    @property
+    def attributes(self) -> List[Attribute]:
+        return [Attribute.from_cli_args(*cli_args) for cli_args in self.obj.get("attribute", [])]
 
     @property
     def xed_info(self) -> XedInfo:
@@ -117,34 +216,30 @@ class MemfaultCliClickContext(object):
 
     @property
     def file_url(self) -> str:
-        FILES_BASE_URL = "https://files.memfault.com"
         url = self.obj.get("url")
         if url is None:
-            return FILES_BASE_URL
+            return "https://files.memfault.com"
         return url
 
     @property
     def app_url(self) -> str:
-        APP_BASE_URL = "https://app.memfault.com"
         url = self.obj.get("url")
         if url is None:
-            return APP_BASE_URL
+            return "https://app.memfault.com"
         return url
 
     @property
     def chunks_url(self) -> str:
-        CHUNKS_BASE_URL = "https://chunks.memfault.com"
         url = self.obj.get("url")
         if url is None:
-            return CHUNKS_BASE_URL
+            return "https://chunks.memfault.com"
         return url
 
     @property
     def api_url(self) -> str:
-        CHUNKS_BASE_URL = "https://api.memfault.com"
         url = self.obj.get("url")
         if url is None:
-            return CHUNKS_BASE_URL
+            return "https://api.memfault.com"
         return url
 
     @property
