@@ -21,7 +21,7 @@ from urllib3 import Retry
 
 from . import functools_ext
 from .authenticator import Authenticator
-from .context import MemfaultCliClickContext, PackageInfo
+from .context import DeltaReleaseVersion, MemfaultCliClickContext, PackageInfo
 from .gnu_build_id import get_gnu_build_id
 
 LOG = logging.getLogger(__name__)
@@ -224,15 +224,18 @@ class XedUploader(Uploader):
 
     def entity_url(self) -> str:
         if self.authenticator.project_key_auth():
-            return f"{self._api_base_url()}/upload/qc/xed"
+            return f"{self._api_base_url()}/upload/qc/x"
         else:
-            return f"{self._projects_base_url()}/qc/xed"
+            return f"{self._projects_base_url()}/qc/x"
 
     def post_token_extra_request_data(self) -> Optional[Dict]:
         info = self.ctx.xed_info
         return {
             "device_serial": info.device_serial,
             "hardware_version": info.hardware_version,
+            "trace_attributes": [
+                {"string_key": a.string_key, "value": a.value} for a in self.ctx.attributes
+            ],
         }
 
 
@@ -317,11 +320,14 @@ class McuSdkElfSymbolUploader(Uploader):
         return f"{self._projects_base_url()}/symbols"
 
     def ui_url(self) -> Optional[str]:
-        if not self.ctx.software_info:
+        if not self.ctx.software_info or (
+            # Unreachable with our CLI, but for safety:
+            isinstance(self.ctx.software_info.version, DeltaReleaseVersion)
+        ):
             return None
 
         software_type = url_quote(self.ctx.software_info.software_type)
-        software_version = url_quote(self.ctx.software_info.software_version)
+        software_version = url_quote(self.ctx.software_info.version.software_version)
         return (
             f"{self._projects_ui_base_url()}/software/{software_type}/versions/{software_version}"
         )
@@ -333,7 +339,7 @@ class McuSdkElfSymbolUploader(Uploader):
         info = self.ctx.software_info
         return {
             "software_version": {
-                "version": info.software_version,
+                "version": info.version.to_json_representable(),
                 "software_type": info.software_type,
                 **({"revision": info.revision} if info.revision else {}),
             },
@@ -353,19 +359,34 @@ class ReleaseArtifactUploader(Uploader):
         return f"{self._projects_base_url()}/releases/ota_payload"
 
     def ui_url(self) -> str:
-        if not self.ctx.software_info or self.ctx.software_info.software_type == "android-build":
-            # In case of Android, the created Release.version may not be the same as the SoftwareVersion.version, in
-            # case build fingerprints are used, the Release.version will be "stripped" (i.e. */*/*:*/*/*:*/*::1.0.0):
-            return f"{self._projects_ui_base_url()}/releases"
-        software_version = url_quote(self.ctx.software_info.software_version)
-        return f"{self._projects_ui_base_url()}/releases/{software_version}"
+        info = self.ctx.software_info
+        releases_url = f"{self._projects_ui_base_url()}/releases"
+        delta_releases_url = f"{self._projects_ui_base_url()}/delta-releases"
+
+        if not info:
+            return releases_url
+
+        is_delta = isinstance(info.version, DeltaReleaseVersion)
+
+        if info.software_type == "android-build":
+            # In case of Android, the created Release.version may not be the same
+            # as the SoftwareVersion.version, in case build fingerprints are used,
+            # the Release.version will be "stripped" (i.e. */*/*:*/*/*:*/*::1.0.0):
+            return delta_releases_url if is_delta else releases_url
+
+        if is_delta:
+            from_version = url_quote(info.version.delta_from)
+            target_version = url_quote(info.version.delta_target)
+            return f"{delta_releases_url}/{from_version}/{target_version}"
+
+        return f"{releases_url}/{url_quote(info.version.software_version)}"
 
     def post_token_extra_request_data(self) -> Optional[Dict]:
         info = self.ctx.software_info
 
         return {
             "software_version": {
-                "version": info.software_version,
+                "version": info.version.to_json_representable(),
                 "software_type": info.software_type,
                 **({"revision": info.revision} if info.revision else {}),
             },
